"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withSimpleCaching = void 0;
const isAPromise_1 = require("./isAPromise");
const serialize_1 = require("./serialize");
/**
 * caches the promise of each invocation, based on serialization of the inputs.
 *
 * by default, this uses the `simple-in-memory-cache`, but you can pass in a `get` and `set` method for any persistent storage (e.g., local storage, cookie storage, dynamodb, etc)
 *
 * for example:
 * ```ts
 * const getApiResult = withSimpleCaching(({ name, number }) => axios.get(URL, { name, number }));
 * const result1 = getApiResult({ name: 'casey', number: 821 }); // calls the api, puts promise of results into cache, returns that promise
 * const result2 = getApiResult({ name: 'casey', number: 821 }); // returns the same promise from above, because it was found in cache - since same input as request above was used
 * expect(result1).toBe(result2); // same exact object - the promise
 * expect(await result1).toBe(await result2); // same exact object - the result of the promise
 * ```
 */
exports.withSimpleCaching = (logic, { cache }) => {
    return ((...args) => __awaiter(void 0, void 0, void 0, function* () {
        // define key based on args the function was invoked with
        const key = serialize_1.serialize({ args });
        // see if we already have this result cached
        const cached = cache.get(key);
        if (cached)
            return cached; // return the value if its already cached
        // if we dont, then grab the result of the logic
        const valueOrPromise = logic(...args);
        // if the result of the logic was a promise, then return a promise which cache's the result after the promise resolves; // NOTE: we only do this if the result was a promise so we only make the function async if the logic was already async
        if (isAPromise_1.isAPromise(valueOrPromise)) {
            return valueOrPromise.then((value) => {
                cache.set(key, value);
                return value;
            });
        }
        // since the result of the logic was not a promise, then cache the value and return it immediately
        const value = valueOrPromise;
        cache.set(key, value);
        return value;
    }));
};
//# sourceMappingURL=withSimpleCaching.js.map